# Static HTML Node

## Overview

The `Static HTML Node` module significantly enhances the performance of your website by generating and serving static HTML versions of heavy pages, reducing server load and improving page load times for users.

## Features

- **Static HTML Generation:** Generates static HTML pages for individual nodes.
- **Automatic Redirect:** Redirects users to the static page when available.
- **Intelligent Cache Handling:** Automatically clears relevant caches when static pages are created or deleted.
- **Homepage Support:** Generate and serve static versions of your site's homepage.
- **File Management:** Static pages are stored as files in a configurable folder (default: `public://static`), making it easy to manage. The folder can be changed in the module settings form.
- **File List Page:** Provides an admin page (`/admin/content/static-files`) to view, manage, and delete generated static files. Each static file is related to its source node.
- **Node Form Integration:** Adds a button to the node edit form to manually generate or update static pages.
- **Bulk Node Actions Integration:** Adds bulk actions to the content listing page to generate or delete static content for multiple nodes at once.
- **Operation Actions:** Adds contextual operations for generating and deleting static files directly from the file list page.
- **Permission Control:** Granular permissions for generating and deleting static pages.
- **Node Type Settings:** Allows administrators to select which node types are eligible for static generation in the module settings form.
- **Asset Management:** Automatically handles CSS, JavaScript, images, and other assets required by the static page.

## Requirements

- Drupal 10 or 11
- Dependencies:
  - `drupal:node`
  - `drupal:file`
- PHP 8.0+ (recommended)

## Installation

1. **Download and Install the Module:**
   Place the module in your `modules/custom` directory.

2. **Enable the Module:**
   ```bash
   drush en static_node -y
   ```

3. **Configure Permissions:**
   Visit `admin/people/permissions` and set the following permissions:
   - `generate static node` - Allow users to generate static HTML versions of nodes
   - `delete static node` - Allow users to delete static HTML versions of nodes

## Configuration

Once the module is installed, you can configure it via the following steps:

1. **User Permissions:**
   - Assign the "Generate static node" permission to roles that should be able to create static pages
   - Assign the "Delete static node" permission to roles that should be able to remove static pages

2. **Node Type Settings:**
   - Go to the module settings form (`/admin/config/system/static-node`)
   - Select which node types are eligible for static generation
   - Configure the static folder path (default: `public://static`). All static files will be generated in this folder.

3. **File List Page:**
   - Visit `/admin/content/static-files` to view all generated static files
   - Each file is linked to its source node (if available)
   - Use operation links to delete or regenerate static files

4. **Node Form Alterations:**
   The module adds a "Generate Static Page" button on the node edit form. This button allows you to manually generate or update the static version of the page.

5. **Static Page Deletion:**
   If you want to delete the static version of a page, the node form also provides an option to remove the generated static page.

6. **Homepage Configuration:**
   - Go to Site Information settings (`/admin/config/system/site-information`)
   - Find the "Static Homepage" section
   - Use the buttons to generate or delete a static version of your homepage

## How It Works

1. **Static Page Generation:**
   The static page is generated when the "Generate Static Page" button is clicked or via bulk actions. The module fetches the HTML content of the node and stores it as a `.html` file in the configured static folder.

2. **Asset Processing:**
   The module automatically processes and stores all required assets:
   - CSS files
   - JavaScript files
   - Font files
   - JSON data files
   - Images and icons

3. **Static Page Redirection:**
   If a static page exists, anonymous users will be automatically redirected to the static version. This redirection improves performance by serving static content instead of dynamically generating the page on every request.

4. **Cache Management:**
   When creating or updating static pages, the module automatically clears relevant caches to ensure that changes take effect immediately without requiring manual cache clearing.

5. **Page Deletion:**
   Static pages can be deleted manually from the node form, from the file list page, or via bulk actions. This also clears appropriate caches.

6. **Bulk Operations:**
   The module provides bulk operations that you can use from the content listing page:
   - **Generate Static Page**: Create static versions for multiple nodes at once
   - **Delete Static Page**: Remove static versions for multiple nodes at once

7. **File List Page:**
   - View all generated static files and their related nodes
   - Delete or regenerate static files directly from the list

8. **Configurable Static Folder:**
   - The folder where static files are generated is configurable in the module settings form
   - All file generation, lookup, and deletion operations use this configured folder path

## Example Usage

1. **Generate a Static Page:**
   - Edit a node and click on the "Generate Static Page" button.
   - The static page will be created and stored in the configured static folder.
   - Caches are automatically cleared to ensure the redirect works immediately.

2. **Generate a Static Homepage:**
   - Go to Site Information settings (`/admin/config/system/site-information`)
   - Click "Generate Static Homepage"
   - The static version of your homepage will be created

3. **Access Static Page:**
   - When visiting the node URL as an anonymous user, they will be redirected to the generated static HTML page (if it exists).

4. **Delete Static Page:**
   - Edit the node again and click the "Delete Static Page" button to remove the generated file.
   - Or use the file list page or bulk actions to delete static files.

5. **Bulk Generate Static Pages:**
   - Go to the Content admin page (`/admin/content`)
   - Select multiple nodes using the checkboxes
   - Select "Generate static page" from the "Action" dropdown
   - Click "Apply to selected items"
   - Confirm the action

6. **Manage Static Files:**
   - Go to `/admin/content/static-files` to view, delete, or regenerate static files
   - See which node each static file is related to

## Troubleshooting

### Common Issues

- **Error: Unable to create directory.**
  - Ensure that the configured static directory is writable by the web server.
  - Check the static folder path in the module settings form.
  
- **Static page not redirecting:**
  - Ensure that the static page exists in the correct directory.
  - Try clearing cache: `drush cr`
  - Verify you're testing as an anonymous user (authenticated users always see dynamic pages)
  
- **Permission denied errors:**
  - Check that users have been assigned the proper permissions to generate or delete static pages.

### Known Issues

- The static page is not updated automatically when the content of the node is updated. Updates to the static page will only occur through manual regeneration or saving static node edit form.

## Maintainers

- Alberto Cocchiara (bigbabert) - https://www.drupal.org/u/bigbabert

## License

This module is licensed under the GPLv2 license. See [LICENSE.txt](LICENSE.txt) for more details.