<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\StaticNodeHooks;
use Drupal\Core\Cache\Cache;

/**
 * Provides a confirmation form for generating a static homepage.
 */
class StaticHomepageGenerateForm extends ConfirmFormBase {

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * Constructs a new StaticHomepageGenerateForm.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(
    MessengerInterface $messenger,
    StaticNodeHooks $static_node_hooks
  ) {
    $this->messenger = $messenger;
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_static_homepage_generate_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to generate a static version of the homepage?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('system.site_information_settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will create a static HTML file for the homepage and set up automatic redirection to improve performance.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Generate');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Generate the static homepage
    $this->staticNodeHooks->generateStaticHomepage();
    
    // Invalidate caches
    Cache::invalidateTags(['config:system.site']);
    
    $form_state->setRedirectUrl($this->getCancelUrl());
  }
}
